<?php
/** This file contains the X509 class.
 *
 * @author Stefan Winter <stefan.winter@restena.lu>
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 *
 * @package Developer
 */

/**
 * This class contains handling functions for X.509 certificates
 *
 * @author Stefan Winter <stefan.winter@restena.lu>
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 *
 * @license http://ticker.eduroam.lu/cat/LICENSE
 *
 * @package Developer
 */
class X509 {

/** transform PEM formed certificate to DER format
 *
 *  @param mixed $pem_data blob of data, which is hopefully a PEM certificate
 *  @return the DER representation of the certificate
 *
 *  @author http://php.net/manual/en/ref.openssl.php (comment from 29-Mar-2007)
 */
public function pem2der($pem_data) {
   $begin = "CERTIFICATE-----";
   $end   = "-----END";
   $pem_data = substr($pem_data, strpos($pem_data, $begin)+strlen($begin));
   $pem_data = substr($pem_data, 0, strpos($pem_data, $end));
   $der = base64_decode($pem_data);
   return $der;
}

public function der2pem($der_data) {
   $pem = chunk_split(base64_encode($der_data), 64, "\n");
   $pem = "-----BEGIN CERTIFICATE-----\n".$pem."-----END CERTIFICATE-----\n";
   return $pem;
}
/**
  * prepare PEM and DER formats, MD5 and SHA1 fingerprints and subject of the certificate
  *
  * returns an array with the following fields:
  * <pre> uuid
  * pem	certificate in PEM format
  * der	certificate in DER format
  * md5	MD5 fingerprint
  * sha1	SHA1 fingerprint
  * name	certificate subject
  * </pre>
  * @param blob $cadata certificate in ether PEM or DER format
  * @return array
  */
public function processCertificate ($cadata) {
   if( substr($cadata,0,27) == "-----BEGIN CERTIFICATE-----" ) {
        $ca_pem = $cadata;
        $ca_der = X509::pem2der($ca_pem);
    } else {
        $ca_der = $cadata;
        $ca_pem =  X509::der2pem($cadata);
    }

    # check that the certificate is OK
    $myca = openssl_x509_read($ca_pem);
    if ($myca == FALSE) return FALSE;
    $mydetails = openssl_x509_parse($myca);
    $md5 = openssl_digest($ca_der,'MD5');
    $sha1 = openssl_digest($ca_der,'SHA1');
    $out = array ("uuid" => uuid(), "pem" => $ca_pem, "der" => $ca_der, "md5"=>$md5, "sha1"=>$sha1, "name"=>$mydetails['name']);
    return $out;
}

public function downloadCA($url) {
    $cadata = downloadFile($url);
    if(! $cert = X509::processCertificate($cadata))
              error("The certificate seems broken");
            return($cert["pem"]);
}

}

?>
