<?php
/**
 * This is the collection of methods dedicated for the user GUI
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 * @package UserGUI
 */

require_once("Profile.php");
require_once("Federation.php");
require_once("DeviceFactory.php");
require_once("devices/devices.php");

/**
 * The basic methoods for the user GUI
 * @package UserGUI
 *
 */
class GUI {
/**
 * Prepare the device module environment and send back the link
 * This method creates a device module instance fia the {@link DeviceFactory} call, 
 * then sets up the device module environment for the specific profile by calling 
 * {@link DeviceConfig::setup()} method and finally, called the devide writeInstaller meethod
 * passing the returned path name.
 * {@source}
 * 
 * @param string $device identifier as in {@link devices.php}
 * @param int $prof_id profile identifier
 * @param string $lang optional language identifier as in {@link Config::$LANGUAGES}
 *
 * @return array 
 * array with the following fields:
 * - profile - the profile identifier
 * - system - the device identifier
 * - link - the path name of the resulting installer
 *
 * @todo implement device module cache
 */
  public function generateInstaller($device,$prof_id,$lang = 0) {
    set_locale("devices",$lang);
    debug(4,"installer:$device:$prof_id\n");
    $profile = new Profile($prof_id);
    $a = array();
    $factory = new DeviceFactory($device);
    $dev = $factory->device;
    $a['profile'] = $prof_id;
    $a['system'] = $device;
    if(isset($dev)) {
       $dev->setup($profile);
       $a['link'] = $dev->FPATH.'/'.$dev->writeInstaller();
    } 
    return($a);
 }

/**
 * Prepare the support data for a given profile
 *
 * @param int $prof_id profile identifier
 * @return array
 * array with the following fields:
 * - local_email
 * - local_phone
 * - local_url
 * - devices - an array of device names and their statuses (for a given profile)
 */
 public function profileAttributes($prof_id) {
      $profile = new Profile($prof_id);
      $attr = $profile->getCollapsedAttributes();
      $a = array();
      if(isset($attr['support:email']))
         $a['local_email'] = $attr['support:email'][0];
      if(isset($attr['support:phone']))
         $a['local_phone'] = $attr['support:phone'][0];
      if(isset($attr['support:url']))
         $a['local_url'] = $attr['support:url'][0];
      $a['devices'] = $profile->listDevices();
      return($a);
 }

}
/**
 * JSON methods are the inteface between the {@link GUI} methods and the {@link cat_back.php} backend
 * @package UserGUI
 */
class JSON {

/**
 * Produce the list of all available identity providers in the DiscoJuice format
 * This is the DiscoJuice metadata producer
 *
 * @return string JSON encoded data
 */
  public function listIdentityProvidersForDisco() {
     $idps = Federation::listAllIdentityProviders();
     for ($i = 0 ; $i < count($idps); $i++ ) {
       $logo =  $idps[$i]['entityID'].'.png';
       if(is_file('../resources/logos/'.$logo)) {
         $idps[$i]['icon'] = $logo;
}
     }
     echo json_encode($idps);
  }

/**
 * Produce a list of profiles available for a given IdP
 *
 * @param int $idp_id the IdP identifier
 * @return string JSON encoded data
 */
  public function listProfiles($idp_id) {
     $idp = new IdP($idp_id);
     $profiles = $idp->listProfiles();
     echo json_encode($profiles);
  }
  
/**
 * Call installer generation and return the link
 *
 * @param string $device identifier as in {@link devices.php}
 * @param int $prof_id profile identifier
 * @return string JSON encoded data
 */
  public function generateInstaller($device,$prof_id) {
    debug(4,"JSON::generateInstaller arguments: $device,$prof_id\n");
    $o = GUI::generateInstaller($device,$prof_id);
    debug(4,"output from GUI::generateInstaller:");
    debug(4,$o);
    debug(4,json_encode($o));
    echo json_encode($o);
 }

/**
 * Produce support data prepared within {@link GUI::profileAttributes()}
 * @return string JSON encoded data
 */
  public function profileAttributes($prof_id) {
    echo json_encode(GUI::profileAttributes($prof_id));
  }

}
?>
